const logger = require('@pubsweet/logger')
const { checkJobStatus } = require('../job-runner')
const Manuscript = require('../xpub-model/entities/manuscript/data-access')
const {
  gManuscript,
} = require('../xpub-model/entities/manuscript/helpers/transform')
const { errorDevEmail } = require('../email')
const { createGrantLinks } = require('./linkGrants')

if (!process.env.ENABLE_CRONJOB_GRANTLINKCHECK) {
  logger.info(
    'ENABLE_CRONJOB_GRANTLINKCHECK not defined. grantLinkCheck cronjob exits.',
  )
  process.exit(0)
}

;(async () => {
  await checkJobStatus('link-grants', async () => {
    const updateTime = Date.now()
    logger.info('Sending grant data')
    await grantLinkCheck()
    logger.info(
      `Grant link check and sending was completed in ${Date.now() -
        updateTime} ms`,
    )
  })
  await Manuscript.knex().destroy()
  process.exit(0)
})()

process
  .on('uncaughtException', async (err, origin) => {
    logger.error('Uncaught Exception thrown: ', err)
    logger.error('Exception thrown at: ', origin)
    await Manuscript.knex().destroy()
    process.exit(1)
  })
  .on('unhandledRejection', async (reason, promise) => {
    logger.error('Unhandled Rejection: ', reason)
    logger.error('Rejected at promise: ', promise)
    await Manuscript.knex().destroy()
    process.exit(1)
  })

async function grantLinkCheck() {
  try {
    const linkingManuscripts = await Manuscript.query()
      .where('ncbiState', 'linking grants')
      .whereNull('manuscript.deleted')
    if (linkingManuscripts.length > 0) {
      const grantLinkDone = await createGrantLinks(
        linkingManuscripts.map(manuscript => gManuscript(manuscript)),
      )
      if (!grantLinkDone.success) {
        throw new Error(grantLinkDone.message)
      }
      await Promise.all(
        linkingManuscripts.map(async m => {
          m.ncbiState = 'success'
          await m.save()
          return true
        }),
      )
      return true
    }
    logger.info('No grant links to send.')
    return true
  } catch (e) {
    await errorDevEmail('sending grants to EBI', e)
    logger.error(`Unable to get or send grant links: ${e}`)
    throw e
  }
}
