const Client = require('ftp')
const logger = require('@pubsweet/logger')
const config = require('config')
const fs = require('fs')
const async = require('async')
const { execSync } = require('child_process')
const ftpUser = require('../utils/ftpUser')
const { errorDevEmail } = require('../email')
const { checkJobStatus } = require('../job-runner')
const { FtpAccountManager } = require('../xpub-model')

if (!process.env.ENABLE_CRONJOB_FROMFTPBULKUPLOAD) {
  logger.info(
    'ENABLE_CRONJOB_FROMFTPBULKUPLOAD not defined. fromFtp-BulkUpload cronjob exits.',
  )
  process.exit(0)
}

const BULK_UPLOAD_FILE_EXT = new RegExp(/\S+\.tar\.gz$/i)

// Create queue
const queue = async.queue(async user => {
  logger.info(
    `Bulk Upload: Initialising job for file download from FTP folder: ${user.username}`,
  )
  const ftp = new Client()

  const rootFolderLocal = `${process.env.HOME}/${config.get('ftp_directory')}`

  fs.access(rootFolderLocal, error => {
    if (error) {
      logger.error(`${rootFolderLocal} directory does not exist. ${error}`)
      process.exit(0)
    }
  })

  // create local directories
  const rootPathLocal = `${rootFolderLocal}/${user.username}`
  const cmd = `mkdir -p ${rootPathLocal}`
  execSync(cmd)

  const beforeUpdate = Date.now()

  try {
    const val = await checkNewFtpPackage(ftp, user, rootPathLocal)
    logger.info(
      `Bulk Upload: ${val} file(s) downloaded to local directory ${rootPathLocal} in ${Date.now() -
        beforeUpdate} ms`,
    )
  } catch (err) {
    errorDevEmail('Bulk Upload Cron Job: ', err)
    logger.error(err)
  }
  close(ftp)
  return true
})

// Check and run cronjob
;(async () => {
  await checkJobStatus(
    'from-ftp-bulkupload',
    async () => {
      const ftpBulkUploaders = await FtpAccountManager.findBulkUploaders()
      if (!ftpBulkUploaders) {
        logger.info('No FTP bulk upload users in database. Exiting.')
        return false
      }
      logger.info('Queueing bulk upload users.')
      ftpBulkUploaders.forEach(async user => {
        queue.push(user)
      })
      await queue.drain()
      logger.info('Cron job completed. Exiting.')
    },
    // FTP healthcheck
    true,
  )
  process.exit(0)
})()

queue.error((err, task) => {
  logger.error(`Error in task ${task}: ${err}`)
  process.exit(1)
})

function close(ftp) {
  ftp.end()
  ftp = null
  logger.info('Ftp connection terminated')
}

process
  .on('uncaughtException', (err, origin) => {
    logger.error('Uncaught Exception thrown: ', err)
    logger.error('Exception thrown at: ', origin)
    process.exit(1)
  })
  .on('unhandledRejection', (reason, promise) => {
    logger.error('Unhandled Rejection: ', reason)
    logger.error('Rejected at promise: ', promise)
    process.exit(1)
  })

async function checkNewFtpPackage(ftp, ftpBulkUploader, rootPathLocal) {
  const rootPathFTP = '/'
  const { host } = config.get('bulk-upload-ftp')
  const { username: user, password } = await ftpUser.getFTPAccount(
    ftpBulkUploader.username,
  )

  return new Promise((resolve, reject) => {
    ftp.on('ready', () => {
      ftp.list(rootPathFTP, false, async (err, list) => {
        if (err) {
          close(ftp)
          reject(err)
        }
        // get array of tar.gz files only
        const targzFiles = list.filter(
          file => BULK_UPLOAD_FILE_EXT.test(file.name) && file.type === '-',
        )
        if (!targzFiles.length) {
          resolve(0) // No files to download to local FTP space
        }
        downloadFtpFiles(targzFiles, ftp, rootPathFTP, rootPathLocal)
          .then(() => resolve(targzFiles.length))
          .catch(err => {
            reject(err.message)
          })
      })
    })
    ftp.on('error', () =>
      reject(new Error('Unable to connect to Bulk Upload FTP')),
    )
    const keepalive = 10000
    const connTimeout = 60000
    const pasvTimeout = 60000
    ftp.connect({ host, user, password, keepalive, connTimeout, pasvTimeout })
  })
}

function downloadFtpFiles(targzFiles, ftp, rootPathFTP, rootPathLocal) {
  return Promise.all(
    targzFiles.map(
      file =>
        new Promise((resolve, reject) => {
          const remoteFilePath = `${rootPathFTP}/${file.name}`
          ftp.get(remoteFilePath, (err, stream) => {
            if (err) {
              logger.error(`${remoteFilePath}: ${err.message}`)
              reject(err.message)
            } else if (!stream) {
              logger.error(`${remoteFilePath}: No FTP stream`)
              reject(err.message)
            } else {
              const path = `${rootPathLocal}/${file.name}`
              const writeStream = stream.pipe(fs.createWriteStream(path))
              writeStream.on('finish', () => {
                logger.info(
                  `Bulk Upload: FTP file has been successfully downloaded to local directory ${path}`,
                )
                resolve()
              })
            }
          })
        }),
    ),
  )
}
