const Prometheus = require('prom-client')
const {
  checkDBStatus,
  getDBStatus,
  getDBStatusDatacenter,
} = require('./postgres-status')
const { checkMinioStatus } = require('./minio-status')
const gridFsClient = require('../gridFs/src/gridFsClient')
const { checkFtpStatus } = require('./ftp-status')
const logger = require('@pubsweet/logger')

module.exports = app => {
  Prometheus.collectDefaultMetrics()

  app.get('/metrics', async (req, res) => {
    await checkDBStatus()
    await checkMinioStatus()
    res.set('Content-Type', Prometheus.register.contentType)
    res.end(Prometheus.register.metrics())
  })

  app.get('/postgres/healthcheck', async (req, res) => {
    const status = await getDBStatus()
    if (status === 1) {
      res.status(200).send('OK')
    } else {
      res.status(500).send('Error')
    }
  })

  app.get('/healthcheck', async (req, res) => {
    const startTime = new Date()
    let status = 'OK'
    let statusPostgres = 'OK'
    let statusFtp = 'OK'
    let statusDb
    let statusMongoDB
    try {
      let { datacenterName } = req.query
      if (datacenterName == null) {
        datacenterName = 'HX'
      }
      statusDb = await getDBStatusDatacenter(datacenterName)
    } catch (err) {
      statusDb = 0
    }

    if (statusDb !== 1) {
      logger.error('HEALTHCHECK:POSTGRES DOWN')
      status = 'KO'
      statusPostgres = 'KO'
    }

    const countMongoDbFiles = await gridFsClient.checkConnection()
    if (countMongoDbFiles < 0) {
      logger.error('HEALTHCHECK:MONGODB DOWN')
      status = 'KO'
      statusMongoDB = 'KO'
    } else {
      statusMongoDB = `Number of files ${countMongoDbFiles}`
    }

    let checkFtp = 1
    const { ftpExcluded } = req.query
    if (ftpExcluded == null || ftpExcluded === '0') {
      checkFtp = await checkFtpStatus()
    }
    if (checkFtp === 0) {
      logger.error('HEALTHCHECK:FTP DOWN')
      status = 'KO'
      statusFtp = 'KO'
    }

    const endTime = new Date()

    const totalTime = endTime - startTime

    const checkingInfo = {
      status,
      statusMongoDB,
      statusPostgres,
      statusFtp,
      totalTime,
    }

    if (status === 'OK') {
      res.status(200).json(checkingInfo)
    } else {
      res.status(500).json(checkingInfo)
    }
  })
}
