const logger = require('@pubsweet/logger')
const config = require('config')
const Audit = require('../xpub-model/entities/audit/data-access')
const { ref } = require('objection')
const { checkJobStatus } = require('../job-runner')
const { userMessage } = require('../email')

if (!process.env.ENABLE_CRONJOB_TAGGINGALERT) {
  logger.info(
    'ENABLE_CRONJOB_TAGGINGALERT not defined. taggingAlert cronjob exits.',
  )
  process.exit(0)
}

const { externalQA, system } = config['epmc-email']
const ftpTagger = config.get('ftp_tagger')

;(async () => {
  await checkJobStatus('tagging-alert', async () => {
    const updateTime = Date.now()
    logger.info(`Counting tagged manuscripts`)
    await taggingAlert()
    await qaReceipt()
    logger.info(`Alerts were completed in ${Date.now() - updateTime} ms`)
  })
  await Audit.knex().destroy()
  process.exit(0)
})()

process
  .on('uncaughtException', async (err, origin) => {
    logger.error('Uncaught Exception thrown: ', err)
    logger.error('Exception thrown at: ', origin)
    await Audit.knex().destroy()
    process.exit(1)
  })
  .on('unhandledRejection', async (reason, promise) => {
    logger.error('Unhandled Rejection: ', reason)
    logger.error('Rejected at promise: ', promise)
    await Audit.knex().destroy()
    process.exit(1)
  })

async function taggingAlert() {
  try {
    const tagging = await Audit.query()
      .aliasFor(Audit, 'a')
      .select('a.manuscriptId', 'a.manuscriptVersion', 'organization.name')
      .where('a.objectType', 'manuscript')
      .whereRaw(
        "a.\"created\"::date = current_date - interval '1 day' and a.changes->>'status' = 'tagging'",
      )
      .joinRelation('manuscript')
      .whereNull('manuscript.deleted')
      .distinct('a.manuscriptId')
      .join('organization', 'manuscript.organizationId', 'organization.id')
    logger.info(`${tagging.length} manuscripts sent to tagging yesterday.`)
    if (tagging.length > 0) {
      logger.info('Sending tagging count email.')
      await sendTaggingMessage(tagging)
      return true
    }
  } catch (e) {
    logger.error(`Unable to get or send tagging count: ${e}`)
  }
  return true
}

async function qaReceipt() {
  try {
    const qa = await Audit.query()
      .aliasFor(Audit, 'a')
      .select(
        'a.manuscriptId',
        ref('a.changes:status')
          .castText()
          .as('status'),
        'a.manuscriptVersion',
        'organization.name',
      )
      .where('a.objectType', 'manuscript')
      .whereRaw(
        "a.\"created\"::date = current_date - interval '1 day' and a.changes->>'status' in ('xml-review', 'xml-triage', 'repo-ready') and a.original_data->>'status' = 'xml-qa'",
      )
      .joinRelation('manuscript')
      .whereNull('manuscript.deleted')
      .distinct('a.manuscriptId')
      .join('organization', 'manuscript.organizationId', 'organization.id')
    logger.info(`${qa.length} manuscripts moved through QA yesterday.`)
    if (qa.length > 0) {
      logger.info('Sending QA receipt email.')
      await sendQAMessage(qa)
      return true
    }
  } catch (e) {
    logger.error(`Unable to get or send QA count: ${e}`)
  }
  return true
}

async function sendTaggingMessage(list) {
  const listM = list.filter(m => m.name === 'Europe PMC Plus')
  const listP = list.filter(m => m.name === 'Europe PMC Preprints')
  const message = `${list.length} manuscript${
    list.length === 1 ? ' was' : 's were'
  } sent for tagging yesterday.${
    listM.length > 0
      ? `\n\n<b>Author manuscripts</b>
${listM.map(m => m.manuscriptId).join('\n')}`
      : ''
  }${
    listP.length > 0
      ? `\n\n<b>Preprints</b>
${listP
  .map(m => `${m.manuscriptId} v${Math.round(m.manuscriptVersion)}`)
  .join('\n')}`
      : ''
  }`
  const subject = `${list.length} submissions sent for tagging`
  await userMessage(externalQA, subject, message, null, ftpTagger.email, system)
  return true
}

async function sendQAMessage(list) {
  const listM = list.filter(m => m.name === 'Europe PMC Plus')
  const listP = list.filter(m => m.name === 'Europe PMC Preprints')
  const listME = listM.filter(m => m.status === 'xml-triage')
  const listMA = listM.filter(m => m.status !== 'xml-triage')
  const listPE = listP.filter(m => m.status === 'xml-triage')
  const listPA = listP.filter(m => m.status !== 'xml-triage')
  let message = `${list.length} manuscript${
    list.length === 1 ? '' : 's'
  } went through QA yesterday. Summary:\n\n<table cellpadding="5px" frame="void" rules="all" style="width: 100%; text-align: left; color: #494949; line-height: 150%">
  <thead>
    <tr style="text-align: center">
      <th colspan="2">Author manuscripts (${listM.length})</th>
      <th colspan="2">Preprints (${listP.length})</th>
    </tr>
    <tr>
      <th>Errors (${listME.length})</th><th>Approved (${listMA.length})</th>
      <th>Errors (${listPE.length})</th><th>Approved (${listPA.length})</th>
    </tr>
  </thead>
  <tbody><tr>`
  message += `<td valign="top">${
    listME.length > 0 ? listME.map(m => m.manuscriptId).join('<br/>') : 'None'
  }</td>`
  message += `<td valign="top">${
    listMA.length > 0 ? listMA.map(m => m.manuscriptId).join('<br/>') : 'None'
  }</td>`
  message += `<td valign="top">${
    listPE.length > 0
      ? listPE
          .map(m => `${m.manuscriptId} v${Math.round(m.manuscriptVersion)}`)
          .join('<br/>')
      : 'None'
  }</td>`
  message += `<td valign="top">${
    listPA.length > 0
      ? listPA
          .map(m => `${m.manuscriptId} v${Math.round(m.manuscriptVersion)}`)
          .join('<br/>')
      : 'None'
  }</td>`
  message += `</tr></tbody></table>`
  const subject = `QA receipt: ${list.length} submissions`
  await userMessage(externalQA, subject, message, null, null, system)
  return true
}
