const { Model } = require('objection')
const EpmcBaseModel = require('../epmc-base-model')

const BaseModel = require('@pubsweet/base-model')

const knex = BaseModel.knex()

class Audit extends EpmcBaseModel {
  static get tableName() {
    return 'audit.audit_log'
  }

  static get schema() {
    return {
      properties: {
        id: { type: 'uuid' },
        created: { type: 'timestamp' },
        userId: { type: 'uuid' },
        action: { type: 'string' },
        objectId: { type: 'uuid' },
        originalData: { type: 'object' },
        changes: { type: 'object' },
        objectType: { type: 'string' },
        manuscriptId: { type: 'string' },
        manuscriptVersion: { anyOf: [{ type: 'numeric' }, { type: 'null' }] },
      },
    }
  }

  static get relationMappings() {
    const User = require('../user/data-access')
    const Manuscript = require('../manuscript/data-access')
    return {
      user: {
        relation: Model.BelongsToOneRelation,
        modelClass: User,
        join: {
          from: 'audit.audit_log.userId',
          to: 'users.id',
        },
      },
      manuscript: {
        relation: Model.HasOneRelation,
        modelClass: Manuscript,
        join: {
          from: 'audit.audit_log.manuscriptId',
          to: 'manuscript.id',
        },
      },
    }
  }

  static async qaCheck(manId, manVer) {
    const rows = await Audit.query()
      .select('created')
      .where('manuscriptId', manId)
      .where('manuscriptVersion', manVer)
      .whereRaw("original_data->>'status' = 'xml-qa'")
      .whereRaw(
        "changes->>'status' in ('xml-review', 'xml-triage', 'repo-ready')",
      )
    if (rows.length > 0) return true
    return false
  }

  static async publisherMetrics(startMonth, endMonth) {
    const stats = await knex.raw(`
with publishers as (
  select i.user_id, i.meta->>'publisher' as name
  from identity i
  where i.meta->>'publisher' is not null 
)
select p.name, p.user_id, display_mth, created_count as count from (
  select TO_CHAR(date_trunc('month', current_date AT TIME ZONE 'Europe/London') + -1*n * INTERVAL '1 month', 'YYYYMM') mth,
  TO_CHAR(date_trunc('month', current_date AT TIME ZONE 'Europe/London') + -1*n * INTERVAL '1 month', 'Mon YYYY') display_mth
  from generate_series(${startMonth}, ${endMonth}) as n
) dates
full outer join (
  select a.user_id, TO_CHAR(date_trunc('month', a.created AT TIME ZONE 'Europe/London') , 'YYYYMM') mth, count(*) created_count
  from audit.audit_log a where a.object_type='manuscript' and a.changes->>'id' is not null 
  and a.created AT TIME ZONE 'Europe/London' BETWEEN DATE_TRUNC('month', current_date AT TIME ZONE 'Europe/London') + -${endMonth} * INTERVAL '1 month' 
    and DATE_TRUNC('month', current_date AT TIME ZONE 'Europe/London') + -${startMonth -
      1} * INTERVAL '1 month'
  and a.user_id in (select p.user_id from publishers p)
  GROUP BY TO_CHAR(date_trunc('month', a.created AT TIME ZONE 'Europe/London') , 'YYYYMM'), a.user_id
) created on dates.mth = created.mth
right join publishers p on p.user_id = created.user_id
    `)
    const { rows } = stats
    return rows
  }

  static async getMetrics(startMonth, endMonth, orgId) {
    // display_mth, submitted, xml_review, xml_review_within_10_days, xml_review_within_10_days_perc,
    // xml_review_within_3_days, xml_review_within_3_days_perc, published, ncbi_ready_median, external_qa
    // xml_errors, xml_tagging, xml_tagging_within_3_days, xml_tagging_within_3_days_perc
    const metrics = await knex.raw(`
with audits as (
  select au.* from audit.audit_log au
  inner join manuscript m on au.manuscript_id = m.id 
  where m.organization_id='${orgId}'
  and au.created between date_trunc('month', current_date AT TIME ZONE 'Europe/London' - '${endMonth} month'::interval)
    and date_trunc('month', current_date AT TIME ZONE 'Europe/London' - '${startMonth -
      1} month'::interval)
  and (
    au.changes->>'status' is not null or 
    au.changes->>'type' = 'PMC' or 
    au.changes::text ~* '(re-|re)tag'
  )
), external as (
  select * from audits ae where ae.user_id not in (
    select user_id from team t where t.role_name='admin'
  )
  and ae.changes->>'status' in ('xml-review', 'xml-triage', 'repo-ready')
  and ae.original_data->>'status' = 'xml-qa'
), taggers as (
  select aa.manuscript_id, aa.manuscript_version, aa.created, ab.sent
  from (
    select at.manuscript_id, at.manuscript_version, min(at.created) created from audits at 
    where at.user_id in (
      select user_id from team t where t.role_name='tagger'
    )
    and at.changes->>'type' = 'PMC'
    group by at.manuscript_id, at.manuscript_version
    union 
    select a.manuscript_id, a.manuscript_version, a.created created from audits a
    where a.user_id in (
      select user_id from team t where t.role_name='admin'
    )
    and a.changes::text ~* '(re-|re)tag'
    and a.changes->>'notes_type'='userMessage'
  ) aa join (
    select att.manuscript_id, att.manuscript_version, min(att.created) sent from audits att 
    where att.changes->>'status' = 'tagging' and att.original_data->>'status' = 'submitted'
    group by att.manuscript_id, att.manuscript_version
  ) ab on 
    aa.manuscript_id = ab.manuscript_id and aa.manuscript_version = ab.manuscript_version
)
select display_mth, submitted, xml_review, xml_review_within_10_days, 
  CASE WHEN xml_review=0 THEN 0 ELSE xml_review_within_10_days*100/xml_review END xml_review_within_10_days_perc,
  xml_review_within_3_days,
  CASE WHEN xml_review=0 THEN 0 ELSE xml_review_within_3_days*100/xml_review END xml_review_within_3_days_perc,
  published, ncbi_ready_median, external_qa, xml_errors, xml_tagging, xml_tagging_within_3_days,
  CASE WHEN xml_tagging=0 THEN 0 ELSE xml_tagging_within_3_days*100/xml_tagging END xml_tagging_within_3_days_perc
from (
  select TO_CHAR(date_trunc('month', current_date AT TIME ZONE 'Europe/London') + -1*n * INTERVAL '1 month', 'YYYYMM') mth,
    TO_CHAR(date_trunc('month', current_date AT TIME ZONE 'Europe/London') + -1*n * INTERVAL '1 month', 'Mon YYYY') display_mth,
    SUM(CASE WHEN TO_CHAR(d.submitted_date, 'Mon YYYY') = TO_CHAR(date_trunc('month', current_date AT TIME ZONE 'Europe/London') + -1*n.n * INTERVAL '1 month', 'Mon YYYY') THEN 1 ELSE 0 END) submitted,
    SUM(CASE WHEN TO_CHAR(d.xml_review_date, 'Mon YYYY') = TO_CHAR(date_trunc('month', current_date AT TIME ZONE 'Europe/London') + -1*n.n * INTERVAL '1 month', 'Mon YYYY') THEN 1 ELSE 0 END) xml_review,
    SUM(CASE WHEN TO_CHAR(d.xml_review_date, 'Mon YYYY') = TO_CHAR(date_trunc('month', current_date AT TIME ZONE 'Europe/London') + -1*n.n * INTERVAL '1 month', 'Mon YYYY')
      THEN CASE WHEN DATE_PART('DAY', xml_review_date-submitted_date) < 11 THEN 1 ELSE 0 END
      ELSE 0 END) xml_review_within_10_days,
    SUM(CASE WHEN TO_CHAR(d.xml_review_date, 'Mon YYYY') = TO_CHAR(date_trunc('month', current_date AT TIME ZONE 'Europe/London') + -1*n.n * INTERVAL '1 month', 'Mon YYYY')
      THEN CASE WHEN DATE_PART('DAY', xml_review_date-submitted_date) < 4 THEN 1 ELSE 0 END
      ELSE 0 END) xml_review_within_3_days,
    SUM(CASE WHEN TO_CHAR(d.first_published_date, 'Mon YYYY') = TO_CHAR(date_trunc('month', current_date AT TIME ZONE 'Europe/London') + -1*n.n * INTERVAL '1 month', 'Mon YYYY') THEN 1 ELSE 0 END) published,
    percentile_cont(0.5) within group ( order by CASE WHEN TO_CHAR(d.submitted_date, 'Mon YYYY') = TO_CHAR(date_trunc('month', current_date AT TIME ZONE 'Europe/London') + -1*n.n * INTERVAL '1 month', 'Mon YYYY') THEN DATE_PART('DAY', ncbi_ready_date-submitted_date) ELSE NULL END) ncbi_ready_median
  from generate_series(${startMonth}, ${endMonth}) n, audit.manuscript_process_dates d 
  inner join manuscript m on d.manuscript_id = m.id where m.organization_id='${orgId}'
  group by TO_CHAR(date_trunc('month', current_date AT TIME ZONE 'Europe/London') + -1*n.n * INTERVAL '1 month', 'Mon YYYY'), n.n
) as top
full outer join (
  select TO_CHAR(date_trunc('month', ext.created AT TIME ZONE 'Europe/London') , 'YYYYMM') mth, count(*) external_qa,
  SUM(CASE WHEN ext.changes->>'status' = 'xml-triage' THEN 1 ELSE 0 END) xml_errors
  from external ext
  group by mth
) qa on top.mth = qa.mth
full outer join (
  select TO_CHAR(date_trunc('month', tag.created AT TIME ZONE 'Europe/London') , 'YYYYMM') mth, count(*) xml_tagging,
  SUM(CASE WHEN DATE_PART('DAY', tag.created-tag.sent) < 4 THEN 1 ELSE 0 END) xml_tagging_within_3_days
  from taggers tag
  group by mth
) tagging on top.mth = tagging.mth
order by top.mth desc
`)

    const { rows } = metrics
    return rows
  }

  static async weeklyMetrics(orgId) {
    const metrics = await knex.raw(`
with audits as (
  select au.* from audit.audit_log au
  inner join manuscript m on au.manuscript_id = m.id WHERE m.organization_id='${orgId}'
  and au.created between date_trunc('week', current_date AT TIME ZONE 'Europe/London') + -4 * '1 week'::interval and current_timestamp AT TIME ZONE 'Europe/London'
  and (
    au.changes->>'status' is not null or 
    au.changes->>'type' = 'PMC' or 
    au.changes::text ~* '(re-|re)tag'
  )
), external as (
  select * from audits ae where ae.user_id not in (
    select user_id from team t where t.role_name='admin'
  )
  and ae.changes->>'status' in ('xml-review', 'xml-triage', 'repo-ready')
  and ae.original_data->>'status' = 'xml-qa'
), taggers as (
  select at.manuscript_id, at.manuscript_version, min(at.created) created from audits at 
  where at.user_id in (
    select user_id from team t where t.role_name='tagger'
  )
  and at.changes->>'type' = 'PMC'
  group by at.manuscript_id, at.manuscript_version
  union 
  select a.manuscript_id, a.manuscript_version, a.created created from audits a
  where a.user_id in (
    select user_id from team t where t.role_name='admin'
  )
  and a.changes::text ~* '(re-|re)tag'
  and a.changes->>'notes_type'='userMessage'
)
select display_mth, submitted, xml_review, published, external_qa, xml_tagging, xml_errors
from (
  select TO_CHAR(date_trunc('week', current_date AT TIME ZONE 'Europe/London') + -1*n * INTERVAL '1 week', 'IYYYIW') wk,
    TO_CHAR(date_trunc('week', current_date AT TIME ZONE 'Europe/London') + -1*n * INTERVAL '1 week', 'Mon DD') display_mth,
    SUM(CASE WHEN TO_CHAR(d.submitted_date, 'IYYYIW') = TO_CHAR(date_trunc('week', current_date AT TIME ZONE 'Europe/London') + -1*n.n * INTERVAL '1 week', 'IYYYIW') THEN 1 ELSE 0 END) submitted,
    SUM(CASE WHEN TO_CHAR(d.xml_review_date, 'IYYYIW') = TO_CHAR(date_trunc('week', current_date AT TIME ZONE 'Europe/London') + -1*n.n * INTERVAL '1 week', 'IYYYIW') THEN 1 ELSE 0 END) xml_review,
    SUM(CASE WHEN TO_CHAR(d.first_published_date, 'IYYYIW') = TO_CHAR(date_trunc('week', current_date AT TIME ZONE 'Europe/London') + -1*n.n * INTERVAL '1 week', 'IYYYIW') THEN 1 ELSE 0 END) published
  from generate_series(0, 4) n, audit.manuscript_process_dates d 
  inner join manuscript m on d.manuscript_id = m.id where m.organization_id='${orgId}'
  group by TO_CHAR(date_trunc('week', current_date AT TIME ZONE 'Europe/London') + -1*n.n * INTERVAL '1 week', 'IYYYIW'), n.n
) as top
full outer join (
  select TO_CHAR(date_trunc('week', ext.created AT TIME ZONE 'Europe/London') , 'IYYYIW') wk, count(*) external_qa,
  SUM(CASE WHEN ext.changes->>'status' = 'xml-triage' THEN 1 ELSE 0 END) xml_errors
  from external ext
  group by wk
) qa on top.wk = qa.wk
full outer join (
  select TO_CHAR(date_trunc('week', tag.created AT TIME ZONE 'Europe/London') , 'IYYYIW') wk, count(*) xml_tagging
  from taggers tag
  group by wk
) tagging on top.wk = tagging.wk
order by top.wk desc
`)

    const { rows } = metrics
    return rows
  }
}

module.exports = Audit
