/* eslint-disable class-methods-use-this */
// method $beforeValidate does not use this
const EpmcBaseModel = require('../epmc-base-model')
const { rowToEntity, buildQuery, runQuery } = require('../util')
const { AjvValidator } = require('objection')

class Job extends EpmcBaseModel {
  static get tableName() {
    return 'config.job'
  }

  static get schema() {
    return {
      properties: {
        name: { type: 'string' },
        description: { type: 'string' },
        running: { type: 'boolean' },
        lastStatus: { type: 'string', enum: ['pass', 'fail'] },
        lastPass: { type: 'timestamp' },
        frequency: {
          type: 'string',
          format: 'regex',
          pattern:
            '(@(annually|yearly|monthly|weekly|daily|hourly|reboot))|(@every (\\d+(ns|us|µs|ms|s|m|h))+)|((((\\d+,)+\\d+|([\\d\\*]+(\\/|-)\\d+)|\\d+|\\*) ?){5,7})',
        },
      },
    }
  }

  // remove inserted updated date
  $beforeUpdate() {
    delete this.updated
  }

  static async selectByName(name, trx) {
    const rows = await Job.query(trx).where({ name })
    if (!rows.length) {
      throw new Error('Job not found')
    }
    return rowToEntity(rows[0])
  }

  static async selectAll() {
    const rows = await runQuery(
      buildQuery
        .select()
        .from('config.job')
        .orderBy('name'),
    )
    return rows.map(rowToEntity)
  }

  static async selectAllNames() {
    const rows = await runQuery(buildQuery.select('name').from('config.job'))
    return rows.map(r => r.name)
  }

  static async update(job, userId, trx) {
    const update = await Job.query(trx)
      .patch(job)
      .where('name', job.name)
      .returning('*')
    const [updated] = update
    return updated
  }
}

class Property extends EpmcBaseModel {
  static get tableName() {
    return 'config.property'
  }

  // Edit validator to allow 'block' format for text blocks in form
  static createValidator() {
    return new AjvValidator({
      onCreateAjv: ajv => {
        ajv.addFormat('block', /[\s\S]+/)
      },
    })
  }

  static get schema() {
    return {
      properties: {
        name: { type: 'string' },
        description: { type: 'string' },
        // value will be checked against schema
        value: true,
        schema: true,
      },
    }
  }

  // Validate value json against schema in schema column
  $beforeValidate(jsonSchema, json, opt) {
    jsonSchema.properties.value = json.schema
    return jsonSchema
  }

  // Remove inserted updated date
  // Ensure JSON arrays are correctly formatted
  $beforeUpdate() {
    this.schema = JSON.stringify(this.schema)
    this.value = JSON.stringify(this.value)
    delete this.updated
  }

  static async selectByName(name, trx) {
    const rows = await Property.query(trx).where({ name })
    if (!rows.length) {
      throw new Error('Config property not found')
    }
    return rowToEntity(rows[0])
  }

  static async selectAll() {
    const rows = await runQuery(
      buildQuery
        .select()
        .from('config.property')
        .orderBy('name'),
    )
    return rows.map(rowToEntity)
  }

  static async update(prop, userId, trx) {
    const update = await Property.query(trx)
      .patch(prop)
      .where('name', prop.name)
      .returning('*')
    const [updated] = update
    return updated
  }
}

module.exports = { Job, Property }
