const uuidv1 = require('uuid/v1')
const config = require('config')
const logger = require('@pubsweet/logger')
const { Readable } = require('stream')
const rfr = require('rfr')
const gridFsClient = require('../../../gridFs/src/gridFsClient')

const { ManuscriptManager, FileManager } = rfr('server/xpub-model')

async function uploadCitation(_, { manuscriptId }, { user }) {
  const manuscript = await ManuscriptManager.findById(manuscriptId, user)
  const { meta } = manuscript
  const {
    articleIds: aIds,
    volume,
    issue,
    location,
    publicationDates,
    citerefUrl,
    fulltextUrl,
  } = meta
  const { fpage, lpage, elocationId } = location
  const articleIds = aIds.filter(aid => aid.pubIdType !== 'pre-pmc')
  const contents = `<citation>${
    volume
      ? `
  <volume>${volume}</volume>`
      : ''
  }${
    issue
      ? `
  <issue>${issue}</issue>`
      : ''
  }${
    fpage
      ? `
  <fpage>${fpage}</fpage>`
      : ''
  }${
    lpage
      ? `
  <lpage>${lpage}</lpage>`
      : ''
  }${
    elocationId
      ? `
  <elocation-id>${elocationId}</elocation-id>`
      : ''
  }${
    articleIds.length > 0
      ? articleIds
          .map(
            aid => `\n  <article-id pub-id-type="${aid.pubIdType}">${aid.id}</article-id>
  `,
          )
          .join('')
      : ''
  }${
    publicationDates.length > 0
      ? publicationDates
          .map(pDate =>
            pDate.jatsDate
              ? `<pub-date pub-type="${
                  elocationId && !fpage && pDate.type === 'ppub'
                    ? 'collection'
                    : pDate.type
                }">\n${Object.keys(pDate.jatsDate)
                  .sort()
                  .map(el =>
                    pDate.jatsDate[el]
                      ? `    <${el}>${pDate.jatsDate[el]}</${el}>\n`
                      : '',
                  )
                  .join('')}  </pub-date>`
              : '',
          )
          .join('\n  ')
      : ''
  }${
    citerefUrl
      ? `
  <citeref-url>${citerefUrl}</citeref-url>`
      : ''
  }${
    fulltextUrl
      ? `
  <fulltext-url>${fulltextUrl}</fulltext-url>`
      : ''
  }
</citation>`
  const filename = `${manuscriptId}.cit`
  const mimeType = 'text/xml'
  const minioLoc = `${uuidv1()}.cit`
  try {
    const stream = new Readable()
    stream._read = () => {}
    stream.push(contents)
    stream.push(null)

    await gridFsClient.storeFileManuscript(
      manuscriptId,
      manuscript.version,
      filename,
      mimeType,
      stream,
      minioLoc,
    )
  } catch (err) {
    logger.error('Minio error: ', err)
    return false
  }
  try {
    const metadataFile = await gridFsClient.getFileMetadata(minioLoc)
    const size = metadataFile.length
    const files = await FileManager.findByManuscriptId(manuscriptId)
    const exists = files.find(f => f.type === 'citation')
    const savedFile = await FileManager.save(
      {
        id: (exists && exists.id) || null,
        manuscriptId,
        filename,
        mimeType,
        type: 'citation',
        url: `${config.file.url.download}/${minioLoc}`,
        size,
      },
      user,
    )
    return !!savedFile
  } catch (err) {
    logger.error('Create file error: ', err)
    return false
  }
}

module.exports = uploadCitation
