const moment = require('moment')
const logger = require('@pubsweet/logger')
const JournalManager = require('../xpub-model/entities/journal/')
const FileManager = require('../xpub-model/entities/file/')
const Manuscript = require('../xpub-model/entities/manuscript/data-access')
const ManuscriptManager = require('../xpub-model/entities/manuscript/')
const getUser = require('../utils/user.js')
const { checkJobStatus } = require('../job-runner')
const config = require('config')
const superagent = require('superagent')
require('superagent-proxy')(superagent)

if (!process.env.ENABLE_CRONJOB_INCOMPLETECITATIONSCHECK) {
  logger.info(
    'ENABLE_CRONJOB_INCOMPLETECITATIONSCHECK not defined. incompleteCitationsCheck cronjob exits.',
  )
  process.exit(0)
}

process
  .on('uncaughtException', (err, origin) => {
    logger.error('Uncaught Exception thrown: ', err)
    logger.error('Exception thrown at: ', origin)
    process.exit(1)
  })
  .on('unhandledRejection', (reason, promise) => {
    logger.error('Unhandled Rejection: ', reason)
    logger.error('Rejected at promise: ', promise)
    process.exit(1)
  })

const eutilsApiKey = config.get('eutils-api-key') || null

let user = null
;(async () => {
  await checkJobStatus('check-incomplete-citations', async () => {
    const beforeUpdate = Date.now()
    logger.info(`Running incomplete citations check...`)
    user = await getUser.getAdminUser()
    await incompleteCitationsCheck()
    logger.info(
      `Incomplete citations check was finished in ${Date.now() -
        beforeUpdate} ms`,
    )
  })
  await Manuscript.knex().destroy()
  process.exit(0)
})()

// module.exports = async publishedCheck => {
async function incompleteCitationsCheck() {
  const manuscripts = await Manuscript.query()
    .where('status', 'xml-complete')
    // .whereNull('meta,volume')
    .whereJsonSupersetOf('meta,article_ids', [{ pubIdType: 'pmid' }])
    .whereNull('manuscript.deleted')

  const pmids = manuscripts
    .map(
      manuscript =>
        manuscript['meta,articleIds'].filter(id => id.pubIdType === 'pmid')[0]
          .id,
    )
    .reduce((acc, pmid) => `${acc + pmid},`, '')

  await checkPubmed(manuscripts, pmids)
}

const isDate = d => {
  const obj = {
    jatsDate: {},
  }
  let date = moment.utc(d, 'YYYY MMM DD')
  if (date.isValid()) {
    obj.jatsDate.day = moment(date).format('DD')
  } else {
    date = moment.utc(d, 'YYYY MMM')
    if (!date.isValid) {
      return false
    }
  }
  obj.jatsDate.year = moment(date).format('YYYY')
  obj.jatsDate.month = moment(date).format('MM')
  obj.date = date.format()
  return obj
}

async function selectResult(results, manuscripts) {
  if (results.result && results.result.uids) {
    await results.result.uids.reduce(async (promise, thisUid) => {
      await promise
      const result = results.result[thisUid]
      const journal = await JournalManager.selectWithNLM(result.nlmuniqueid)
      const publicationDates = []
      if (result.pubdate) {
        const ppub = { type: 'ppub' }
        const pIsDate = isDate(result.pubdate)
        if (pIsDate) {
          ppub.date = pIsDate.date
          ppub.jatsDate = pIsDate.jatsDate
        } else {
          ppub.date = moment.utc(result.pubdate.substring(0, 4), 'YYYY')
          ppub.jatsDate = {
            year: result.pubdate.substring(0, 4),
            season: result.pubdate.substring(5),
          }
        }
        publicationDates.push(ppub)
      }
      if (result.epubdate) {
        const epub = { type: 'epub' }
        const eIsDate = isDate(result.epubdate)
        if (eIsDate) {
          epub.date = eIsDate.date
          epub.jatsDate = eIsDate.jatsDate
        } else {
          epub.date = moment.utc(result.epubdate.substring(0, 4), 'YYYY')
          epub.jatsDate = {
            year: result.epubdate.substring(0, 4),
            season: result.epubdate.substring(5),
          }
        }
        publicationDates.push(epub)
      }
      const pages = result.pages && result.pages.split('-')
      const citationData = {
        journalId: journal.id,
        meta: {
          unmatchedJournal: null,
          title: result.title,
          volume: result.volume,
          issue: result.issue,
          location: {
            fpage: pages ? pages[0] : null,
            lpage: pages ? pages.pop() : null,
            elocationId: result.elocationid,
          },
          publicationDates,
          articleIds: [
            {
              pubIdType: 'pmid',
              id: result.uid,
            },
          ],
        },
      }
      if (
        result.articleids &&
        result.articleids.find(i => i.idtype === 'doi')
      ) {
        citationData.meta.articleIds.push({
          pubIdType: 'doi',
          id: result.articleids.find(i => i.idtype === 'doi').value,
        })
      }
      if (
        result.articleids &&
        result.articleids.find(i => i.idtype === 'pmc')
      ) {
        citationData.meta.articleIds.push({
          pubIdType: 'pmcid',
          id: result.articleids.find(i => i.idtype === 'pmc').value,
        })
      }
      const manuscript = manuscripts.find(m =>
        m['meta,articleIds'].find(
          aid => aid.id === thisUid && aid.pubIdType === 'pmid',
        ),
      )
      if (manuscript) {
        citationData.id = manuscript.id
      }
      try {
        await FileManager.deleteByManIdAndType(
          citationData.id,
          'citation',
          user.id,
        )
        await ManuscriptManager.update(citationData, user.id)
        if (
          manuscript.status === 'xml-complete' &&
          citationData.meta &&
          (citationData.meta.volume || citationData.meta.issue)
        ) {
          citationData.status = 'repo-ready'
          citationData.pdfDepositState = 'ADDING_CITATION'
        }
        await ManuscriptManager.update(citationData, user.id)

        logger.info(
          `Updated incomplete citation for manuscript ${citationData.id}`,
        )
      } catch (err) {
        logger.error(`${citationData.id} unable to update citation`)
        logger.error(err)
        ManuscriptManager.update(
          { id: citationData.id, formState: `Problem with citation: ${err}` },
          user.id,
        )
      }
      return Promise.resolve()
    }, Promise.resolve())
  } else {
    logger.info('No citation results from PubMed')
  }
}

function checkPubmed(manuscripts, pmids) {
  return new Promise((resolve, reject) => {
    const superagentRequest = superagent(
      'GET',
      `https://eutils.ncbi.nlm.nih.gov/entrez/eutils/esummary.fcgi?db=pubmed&id=${pmids}&retmode=json${
        eutilsApiKey ? `&api_key=${eutilsApiKey}` : ''
      }`,
    )
    if (process.env.superagent_http_proxy) {
      superagentRequest.proxy(process.env.superagent_http_proxy)
      logger.info('Proxy configured: ', process.env.superagent_http_proxy)
    }
    superagentRequest.end(async (err, response) => {
      if (err) {
        logger.error(err)
        resolve(false)
      } else {
        await selectResult(response.body, manuscripts)
        resolve(true)
      }
    })
  })
}
